#!/usr/bin/python

import argparse
import platform
import shutil
import sys
import pycurl
import os
from subprocess import call
from StringIO import StringIO
import traceback
import xmltodict

def GetDriverList(remotehost):    
    try:
        buf = StringIO()
        
        url = 'http://' + remotehost + '/api/drivers';
        #print "Querying: " + url
        
        c = pycurl.Curl()
        c.setopt(c.URL, url)
        try:
            c.setopt(c.WRITEDATA, buf)
        except:
            c.setopt(c.WRITEFUNCTION, buf.write)
        c.perform()
        c.close()

        #print "Almost Done..."

        s = buf.getvalue()

        #print s

        result = xmltodict.parse(s)

        try: 
            return [int(result['drivers']['driver']['@port'])]
        except:
            lst = []
            for drv in result['drivers']['driver']:
                lst.append(int(drv['@port']))
                            
            return lst
    except:
        print "Failed To Get ARDI Driver Details"
        traceback.print_exc()
        return []
    
parser = argparse.ArgumentParser(description="Perform Actions on All ARDI Drivers")
parser.add_argument('server',help='The URL of your ARDI instance')
parser.add_argument('mode',help='Choose between \'install\' and \'remove\'')
parser.add_argument('--driver',help='Choose only a specific driver type to act on')

import ctypes, os
try:
    is_admin = os.geteuid() == 0
except AttributeError:
    is_admin = ctypes.windll.shell32.IsUserAnAdmin() != 0
    
if is_admin == False:
    print "Error: This must be run as root/an Administrator"
    sys.exit(0)  

args = parser.parse_args()
    
#Override
server=args.server
mode = args.mode
driver = ""
if args.driver is not None:
    driver = args.driver
    print "Restart Restricted To " + driver + " drivers"

if ((mode != 'remove') and (mode != 'install')):
    print "Invalid Mode Selected"
    sys.exit(-1)

#Get Driver Information
drivers = GetDriverList(server)

if (mode == 'remove'):
    if driver == "":
        if platform.system() == "Windows":
            cmd = "c:\python27\python ../services/consolidator/ardiconsolidators.py " + server + " remove"
            call(cmd, shell=True)
        else:
            cmd = "/opt/ardi/srv/consolidator/ardiconsolidators " + server + " remove"
            call(cmd, shell=True)
            
    for drv in drivers:
        try:
            print "Removing Driver " + str(drv) + "..."
            if platform.system() == "Windows":
                cmd = "c:\python27\python ardirmdrv.py " + str(drv) + " " + server
                if driver != "":
                    cmd += " --ifdriver " + driver
                call(cmd, shell=True)
            else:
                cmd = "/opt/ardi/drivers/ardirmdrv " + str(drv) + " " + server
                if driver != "":
                    cmd += " --ifdriver " + driver
                call(cmd, shell=True)
        except:
            print 'Could Not Remove Driver'

    print "Removal Complete"

if (mode == 'install'):
    if driver == "":
        if platform.system() == "Windows":
            cmd = "c:\python27\python ../services/consolidator/ardiconsolidators.py " + server + " remove"
            call(cmd, shell=True)

            cmd = "c:\python27\python ../services/consolidator/ardiconsolidators.py " + server + " install"
            call(cmd, shell=True)
        else:
            cmd = "/opt/ardi/srv/consolidator/ardiconsolidators " + server + " remove"
            call(cmd, shell=True)
            
            cmd = "/opt/ardi/srv/consolidator/ardiconsolidators " + server + " install"
            call(cmd, shell=True)
        
    for drv in drivers:
        try:
            print "Adding Driver " + str(drv) + "..."
            if platform.system() == "Windows":
                cmd = "c:\python27\python ardiinstdrv.py " + str(drv) + " " + server
                if driver != "":
                    cmd += " --ifdriver " + driver
                call(cmd, shell=True)
            else:
                cmd = "/opt/ardi/drivers/ardiinstdrv " + str(drv) + " " + server
                if driver != "":
                    cmd += " --ifdriver " + driver
                call(cmd, shell=True)
        except:
            print 'Could Not Remove Driver'

    print "Installation Complete"