#!/usr/bin/python

import argparse
import platform
import shutil
import sys
import pycurl
import os
from subprocess import call
from StringIO import StringIO
import traceback
import xmltodict

def SetNSSM(nssmpath,drivername,property,value):
    cmd = "\"" + nssmpath + "\" set \"" + drivername + "\" " + property + " " + value
    call(cmd, shell=True)
    #print cmd

def WindowsInstaller(driver,mode,portno,url):
    print "Installing Driver for Windows..."
    pathtopython = "c:\Python27\python.exe"
    
    is64 = False
    if platform.machine().endswith('64'):
        is64 = True

    basepath = os.path.dirname(os.path.realpath(__file__))
    nssmpath = basepath
    if is64:
        nssmpath = nssmpath + "\\nssm\\win64"
    else:
        nssmpath = nssmpath + "\\nssm\\win32"

    nssmpath = nssmpath + "/nssm.exe"    
    print "Creating NSS Service..."

    drivername = "ARDI " + mode[0:1].upper() + mode[1:] + " " + driver.upper() + " Driver " + str(port)

    #Installing the base service...
    cmd = "\"" + nssmpath + "\" install \"" + drivername + "\" \"" + pathtopython + "\" \"\"\"" + basepath + "\\" + mode + "\\" + driver + "\\" + driver + ".py " + "\"\"\" " + str(portno) + " " + url
    call(cmd, shell=True)
    #print "Running - " + cmd    

    #Redirection of I/O...
    SetNSSM(nssmpath,drivername,"AppStderr","c:\\windows\\temp\\ardi" + driver + "driver" + str(port) + ".log")
    SetNSSM(nssmpath,drivername,"AppStderrCreationDisposition","2")
    SetNSSM(nssmpath,drivername,"AppStdout","c:\\windows\\temp\\ardi" + driver + "driver" + str(port) + ".log")
    SetNSSM(nssmpath,drivername,"AppStdoutCreationDisposition","2")    
    SetNSSM(nssmpath,drivername,"AppRotateFiles","1")
    SetNSSM(nssmpath,drivername,"AppRotateBytes","1048576")

    #Prevent Windows Event-Style Terminations (Useless on Python apps)
    SetNSSM(nssmpath,drivername,"AppStopMethodSkip","6")    
    
    print "Starting NSS Service..."
    
    cmd = "\"" + nssmpath + "\" start \"" + drivername + "\""
    call(cmd, shell=True)
    #print "Running - " + cmd    
    
def LinuxInstaller(driver,mode, portno, url):
    
    print "Installing Driver for Linux..."
    
    basepath = os.path.dirname(os.path.realpath(__file__))
    lxpath = basepath + "/linux"
    
    dtype = ""
    if (mode == "hist"):
        dtype = "hist"
    
    print "Creating Daemon Script..."
    file = open(lxpath + '/daemon.py','r')
    fl = file.read()
    file.close()
    st = str(fl)
    st = st.replace("$DRIVER$",driver)
    st = st.replace("$DRIVERTYPE$",dtype)
    st = st.replace("$PORT$",str(portno))
    st = st.replace("$SERVER$",url)
    finalpath = basepath + "/" + mode + "/" + driver + "/" + str(portno)
    file = open(finalpath,'w')
    file.write(st)
    file.flush()
    file.close()    
    print "Marking File as Executable..."
    call("chmod gu+x " + finalpath, shell=True)
    
    #finalpath = finalpath + " " + str(portno)
    
    print "Creating init.d Script..."
    file = open(lxpath + '/init','r')
    fl = file.read()
    file.close()
    st = str(fl)
    st = st.replace("$DRIVER$",driver)
    st = st.replace("$SERVICENAME$","ardi-" + str(portno))
    st = st.replace("$FULLPATH$",finalpath)
    st = st.replace("$PORT$",str(portno))
    st = st.replace("$SERVER$",url)
    initpath = "/etc/init.d/ardi-" + str(portno)
    file = open(initpath,'w')
    file.write(st)
    file.flush()
    file.close()  
    
    call("chmod gu+x " + initpath,shell=True)
        
    print "Setting Runlevel..."
    call("update-rc.d ardi-" + str(portno) + " defaults", shell=True)
    
    print "Creating Log Files..."
    logpath = "/var/log/ardi/driver-" + str(portno) + ".log"
    call("touch " + logpath, shell=True)
    call("chmod a+r " + logpath, shell=True)
        
    print "Starting Service..."

    call("service ardi-" + str(portno) + " start" , shell=True)
    
def GetDriverDetails(remotehost,portno):    
    try:
        buf = StringIO()
        
        url = 'http://' + remotehost + '/api/driverdetails?driver=' + str(portno)
        #print "Querying: " + url
        
        c = pycurl.Curl()
        c.setopt(c.URL, url)
        try:
            c.setopt(c.WRITEDATA, buf)
        except:
            c.setopt(c.WRITEFUNCTION, buf.write)
        c.perform()
        c.close()

        s = buf.getvalue()

        result = xmltodict.parse(s)

        #print result
        try:
            print "Driver Name: " + result['driverinfo']['name']
            print "Driver Type: " + result['driverinfo']['driver']
            print "Driver Mode: " + result['driverinfo']['mode']

            if (result['driverinfo']['mode'] == 'Live'):
                s = 'L'
            else:
                s = 'H'
            s = s + result['driverinfo']['driver']
        except:
            return ""
                            
        return s
    except:
        return ""

parser = argparse.ArgumentParser(description="Manage ARDI Drivers")
parser.add_argument('port', help='The driver ID from the ARDI web page')
parser.add_argument('server', help='The URL of the ARDI site')
parser.add_argument('--ifdriver',help='Only perform the action if the driver is of the matching type')

args = parser.parse_args()
    
#Override
remotehost="localhost"
if args.server != None:
    remotehost = args.server

server = remotehost

ifdriver = ""
if args.ifdriver is not None:
    ifdriver = args.ifdriver

is_admin = False

import ctypes, os
try:
    is_admin = os.geteuid() == 0
except AttributeError:
    is_admin = ctypes.windll.shell32.IsUserAnAdmin() != 0
    
  
if is_admin == False:
    print "Error: This must be run as root/an Administrator"
    sys.exit(0)        

#Install driver as a service...
port = int(args.port)
driver = GetDriverDetails(remotehost,port)

if driver == "":
    print "Driver Not Found On Specified ARDI Instance"
    sys.exit()

mode = "live"
if (driver[0] == 'H'):
    mode = "hist"

driver = driver[1:]

if ifdriver != "":
    if driver != ifdriver:
        print "Driver does not match given filter"
        sys.exit(0)

print "Installing " + driver + " Driver Service"

if platform.system() == "Windows":
    WindowsInstaller(driver,mode,port,server)
    pass
else:
    LinuxInstaller(driver,mode,port,server)
    pass

print "ARDI Driver Service Added!"