#!/usr/bin/python

import sys
import random
import datetime
from datetime import timedelta
import time
#import dateutil.parser

from ardi.driver import histdriverbase

import traceback

import pycurl
import urllib
import json


try:
    from io import BytesIO
except ImportError:
    from StringIO import StringIO as BytesIO


def fixdate(dt):
	ps = dt.find('.')
	if ps > -1:
		return dt[0:ps]
	return dt

class chockinfo:
    def __init__(self):
        self.weight = 0
        self.north = 0
        self.east = 0
        self.roll = 0
        self.pitch = 0
        self.floor = 0
        self.yaw = 0
        self.id = 0

class profileinfo:
	def __init__(self):
		self.id = 0
		self.stime = 0
		self.etime = 0

class ardihistdriver:
    def __init__(self):            
        self.host = ""
        self.version = "1.5.0"
        self.lookuptable = {}

        self._lwname = ""
        self._east = 0
        self._north = 0
        self._supports = 170
        self._alt = 0
        self._id = 0
        self._mgentry = 0
        self._mgexit = 0
        self._tgentry = 0
        self._tgexit = 0
        self._width = 290

        self._heading = 0

        self._lookup = {}

    def SetAddress(self, addr):
        self.connected = False
        bits = addr.split("^")
        filter = ""
        
        #print "Address: " + addr

        #print "Raw Address"
        self.host = bits[0]
        if len(bits) > 1:
            self.version = bits[1]

        if len(bits) > 2:
            off = bits[2].split(',')
            self._xoffset = float(off[0])
            self._yoffset = float(off[1])
            self._zoffset = float(off[2])
            
        #    self.host = self.host + ':' + bits[1]
        #    self.version = bits[2]

    def Connect(self):
        try:            
            buffer = BytesIO()
            ar = pycurl.Curl()

            for pnt in self.core.pointlist:            
                self._lookup[pnt.address] = pnt.code

            #hst = self.host.split(':')
            #self.host = hst[0]
            #self.version = hst[1]
            
            #if len(hst) > 2:
            #    print "Mapping?"
            #    off = hst[2].split(',')
            #    self._xoffset = off[0]
            #    self._yoffset = off[1]
            #    self._zoffset = off[2]
            
            #if len(hst) > 2:
            #    self.version = hst[2]
            #    self.host = hst[0] + ":" + hst[1]
            #else:
            #    self.version = hst[1]
            #    self.host = hst[0]
                
            self.core.logger.info("Requesting INS Configuration @ " + 'http://' + self.host + '/api/v' + self.version + '/json/panelinfo')

            ar.setopt(ar.URL, 'http://' + self.host + '/api/v' + self.version + '/json/panelinfo')
            ar.setopt(ar.WRITEFUNCTION, buffer.write)
            ar.perform()        

            # HTTP response code, e.g. 200.
            if ar.getinfo(ar.RESPONSE_CODE) == 200:

                data = json.loads(buffer.getvalue())
                #print str(data)
                self._width = data['width']
                self._supports = data['supports']
                self._north = data['northing']
                self._east = data['easting']
                self._heading = data['heading']

                self.core.logger.info("Connected to " + data["name"] + " - " + str(self._supports) + " supports and " + str(self._width) + "m long, heading " + str(self._heading))
                self.core.logger.info("Calibration Point: " + str(self._north) + ", " + str(self._east))
                self.connected = True
                ar.close()
                return True
            else:
                print "Request Failed"
                self.connected = False
                ar.close()
        except Exception as e:
            print "Exception Occured - " + str(e)
            self.connected = False;
            return False
        return True	

    def Disconnect(self):
        self.connected = False
        pass    
        
    def Query(self, points, start,end,function,grain,first):
		print "Running Query"
                self.core.logger.info("Requesting History!")
                #sd = dateutil.parser.parse(start+"+0000")
                #ed = dateutil.parser.parse(end+"+0000")
                
                offset = -time.timezone
                
		sd = datetime.datetime.strptime( start, "%Y-%m-%d %H:%M:%S") + datetime.timedelta(0,offset)
		ed = datetime.datetime.strptime( end, "%Y-%m-%d %H:%M:%S" ) + datetime.timedelta(0, offset)

		self.out = ""

		offset = -1
		minprofile = -9999
		stime = time.mktime(sd.timetuple())
		etime = time.mktime(ed.timetuple())

		self.core.logger.info("Requesting From " + str(start) + " ( " + str(stime) + " ) to " + str(end)) 
		
		#Get a list of profiles to try and find one within a valid date range...
		while minprofile == -9999:
			buffer = BytesIO()
			ar = pycurl.Curl()
			ar.setopt(ar.URL, 'http://' + self.host + '/api/v' + self.version + '/json/profiles/' + str(offset) + '/50')
			ar.setopt(ar.WRITEFUNCTION, buffer.write)
			ar.perform()

			profiles = []
			#stimes

			# HTTP response code, e.g. 200.
			if ar.getinfo(ar.RESPONSE_CODE) == 200:
				data = json.loads(buffer.getvalue())

                                #print str(data)
				#print "Comparing To " + str(comptime)

				for dx in data['profiles']:
					ts = dx[7]
					if ts < etime:
						pr = profileinfo();
						pr.id = dx[0]

                                                #print dx[7]
                                                #print dx[6]
                                                
						etm = time.mktime((datetime.datetime.strptime( dx[6], "%Y-%m-%dT%H:%M:%S" )+datetime.timedelta(0,-time.timezone)).timetuple())
                                                #stm = time.mktime((datetime.datetime.strptime( dx[7], "%Y-%m-%dT%H:%M:%S" )+datetime.timedelta(0,-time.timezone)).timetuple())

                                                #print str(etm)
                                                #print str(dx[7])
                                                
						pr.etime = etm
						pr.stime = dx[7]
						profiles.append(pr)
					else:
						lastprofile = dx[0]
						
					if ts < stime:
						minprofile = dx[0]
						print "Found Profile " + str(dx[0])
						break

			#else:
                        #    break
                        offset -= 50

		et = stime
		pset = ""
		for x in range(0,len(profiles)):
			if x == 0:
				pset = str(profiles[x].id)
			else:
				if x != len(profiles)-1:
					pset += ", " + str(profiles[x].id)
				else:
					pset += " and " + str(profiles[x].id)
			
		self.core.logger.info("This request requires profiles " + str(pset))

		for x in reversed(range(0,len(profiles))):
			st = int(profiles[x].stime)
			if (x > 0):
				et = int(profiles[x-1].stime)
			else:
				et = int(etime)
			self.QueryID(profiles[x].id,st,et,points)

		#print self.out
		return self.out.encode('ascii')

    def QueryID(self,profile,framestart,frameend,points):
        try:
            #print "Running Profile..."
            buffer = BytesIO()
            ar = pycurl.Curl()
            ar.setopt(ar.URL, 'http://' + self.host + '/api/v' + self.version + '/json/cutmodel/' + str(profile))
            ar.setopt(ar.WRITEFUNCTION, buffer.write)
            ar.perform()

            self.core.logger.debug("Requesting Profile " + 'http://' + self.host + '/api/v' + self.version + '/json/cutmodel/' + str(profile))

            #print str(points)
            quicklook = {}
            n=0

            # HTTP response code, e.g. 200.
            if ar.getinfo(ar.RESPONSE_CODE) == 200:
                data = json.loads(buffer.getvalue())
                self.core.logger.info("Profile " + str(data['id']) + " contains " + str(data['profileSize']) + " samples")

                for p in points:
                	quicklook[p] = n
                	n = n + 1

                n=0

                chockset = {}
                for sup in range(0,self._supports+1):
                    chockset[sup] = chockinfo()
                    chockset[sup].id = sup

                indx = 0
                pro = data["profile"]
                while indx < len(pro):
                    support = (pro[indx] / self._width) * (self._supports+1)

                    aweight = support % 1
                    sup = int(support)
                    if sup > self._supports:
                        indx = indx + 8
                        continue
                    chockset[sup].weight += aweight
                    chockset[sup].north += pro[indx+5] * aweight
                    chockset[sup].east += pro[indx+4] * aweight
                    chockset[sup].roll += pro[indx+1] * aweight
                    chockset[sup].pitch += pro[indx+2] * aweight
                    chockset[sup].yaw += pro[indx+3] * aweight
                    chockset[sup].floor += pro[indx+6] * aweight
                    
                    aweight = 1-aweight
                    if (aweight > 0) and (support < self._supports):
                        chockset[sup].weight += aweight
                        chockset[sup].north += pro[indx+5] * aweight
                        chockset[sup].east += pro[indx+4] * aweight
                        chockset[sup].floor += pro[indx+6] * aweight
                        
                    indx = indx + 8
                ar.close()
            else:
                self.core.logger.warning("No Response - Check Driver Version")
                ar.close()
                return

            #print "Profile Data Loaded"
            #print str(quicklook)

            for chkid,chk in chockset.items():
                if chk.weight > 0:
                    chk.north /= chk.weight
                    chk.east /= chk.weight
                    chk.roll /= chk.weight
                    chk.pitch /= chk.weight
                    chk.yaw /= chk.weight
                    chk.floor /= chk.weight
                    
                    chk.north += self._yoffset
                    chk.east += self._xoffset
                    chk.floor += self._zoffset
                    
                    nm = "position[" + str(chkid) + "]"
                    
                    if nm in quicklook:
                        #self.core.UpdateData(self._lookup[nm],str(chk.east) + "," + str(chk.north) + ",0")
                        vid = quicklook[p]
                        #print "Sending " + str(quicklook[p]) + " = " + nm;
                        self.out = self.out + self.core.AddRecord(quicklook[nm],framestart,str(chk.east) + "," + str(chk.north) + "," + str(chk.floor),nm)
                        if framestart != frameend:
                            self.out = self.out + self.core.AddRecord(quicklook[nm],frameend,str(chk.east) + "," + str(chk.north) + "," + str(chk.floor),nm)
                        #print str(chkid) + " : " + str(chk.north) + ", " + str(chk.east)
                        #print str(chkid) + " : " + str(chk.east) + "," + str(chk.north) + ",0"
                        
                    nm = "rotation[" + str(chkid) + "]"
                    if nm in quicklook:
                        vpos = str(chk.pitch) + "," + str(self._heading + chk.yaw) + "," + str(chk.roll)
                        self.out = self.out + self.core.AddRecord(quicklook[nm],framestart,vpos,nm)
                        if framestart != frameend:
                            self.out = self.out + self.core.AddRecord(quicklook[nm],frameend,vpos,nm)
                        #self.core.UpdateData(self._lookup[nm],vpos)
                        #print str(chkid) + " : " + str(chk.roll) + ", " + str(chk.pitch) + "," + str(chk.yaw)
                
        except Exception as ex:
            traceback.print_exc()
            print "Data Aquisition Failed - " + str(ex)
            pass

        return self.out.encode('ascii')

class driverfactory:
    def createinstance(self):
        return ardihistdriver()
            
if __name__ == "__main__":
    sdf = driverfactory()
    base = histdriverbase.historian()
    base.start(sdf)