import sys
import random
import datetime
from datetime import timedelta
import pytz
from tzlocal import get_localzone

from ardi.driver import histdriverbase
#from ardi.driver import sqlhelper
from influxdb import InfluxDBClient

def fixdate(dt):
	ps = dt.find('.')
	if ps > -1:
		return dt[0:ps]
	return dt

class influxaddress:
    def __init__(self, addr):
        self.series = ""
        self.field = ""
        self.tags = {}
        self.condition = ""
        self.address = addr
        self.pvalue = ""
        
        bits = addr.split('|')

        #print str(bits)

        if len(bits)>2:
            self.field = bits[0]
            self.condition = bits[1]
            self.series = bits[2]
            tg = bits[1].split(',')
            for q in tg:
                v = q.split('=')
		if len(v) > 1:
			self.tags[v[0]] = v[1]

class ardihistdriver:
    def __init__(self):        
        self.port = 0
        self.db = None        
        self.host = ""
        self.database = ""
        self.username = ""
        self.password = ""
        self.tz = "local"
        self.lookuptable = {}

    def SetAddress(self, addr):
        self.connected = False
        bits = addr.split(":")
        filter = ""

        #print "Raw Address"

        if len(bits) >= 5:
            self.host = bits[0]
            self.port = bits[1]
            self.database = bits[2]
            self.username = bits[3]
            self.password = bits[4]
            self.table = bits[5]
            
    def Connect(self):
        if self.db is not None:
        	try:
        		self.Disconnect()
        	except:
        		traceback.print_exc()
        		pass
        try:
        	self.core.logger.info("Connecting To " + self.host + ":" + self.port + " / " + self.database)
        	self.db = InfluxDBClient(self.host, self.port,self.username,self.password,self.database)
        	self.connected = True
        	self.core.logger.info("Successfully Connected!")
        	return True
        except:
        	self.core.logger.exception("Unable To Connect")
        	return False		

    def Disconnect(self):
        try:			
        	self.db.close()
        except:
        	pass        
        
    def Query(self, points, start,end,function,grain,first):
		print "Performing '" + function + "' query on " + str(points) + " between " + start + " and " + end + " = " + grain								

		#db = InfluxDBClient("54.206.58.61", 8086,"","","recordings")

		#tz = get_localzone()
		##print dir(tz)
		#naive = tz.localize(datetime.datetime.strptime (start, "%Y-%m-%d %H:%M:%S"),is_dst=None)
		#sd = naive.astimezone (pytz.utc)
		sd = datetime.datetime.strptime (start, "%Y-%m-%d %H:%M:%S")

		sdtx = sd.strftime("%Y-%m-%d %H:%M:%S")

		#naive = tz.localize(datetime.datetime.strptime (end, "%Y-%m-%d %H:%M:%S"),is_dst=None)
		#en = naive.astimezone (pytz.utc)

		en = datetime.datetime.strptime (end, "%Y-%m-%d %H:%M:%S")
		entx = en.strftime("%Y-%m-%d %H:%M:%S")

		secdiff = (en - sd).seconds
		chunksize = 0
		if int(grain) < 0:
			chunksize = secdiff / (-int(grain))
		else:
			chunksize = int(grain)

		#print chunksize

		#sd = datetime.datetime.strptime( start, "%Y-%m-%d %H:%M:%S" )
		#ed = datetime.datetime.strptime( end, "%Y-%m-%d %H:%M:%S" )

		keyset = []
		seriesset = []
		fieldset = []
		lastset = []
		nextset = []

		fn = "MEDIAN"
		if function == "min":
			fn = "MINIMUM"
		if function == "max":
			fn = "MAXIMUM"

		addresses = []
		extragroup = ""

		for p in points:
			ad = influxaddress(p)
			addresses.append(ad)
			if ad.field not in fieldset:
				fieldset.append(ad.field)
			if ad.series == None or ad.series == "":
				ad.series = self.table
			if ad.series not in seriesset:
				seriesset.append(ad.series)
			if ad.condition not in keyset:
				keyset.append(ad.condition)
				for key in ad.tags:
					extragroup += "," + key
				#for key in ad.tags:
				#	fieldset.append('_' + key)
			#for key in ad.tags:
				#if key not in keyset:
				#	keyset[key] = []
				#if ad.tags[key] not in keyset[key]:
				#	keyset[key].append(ad.tags[key])

		#print str(fieldset)
		#print str(seriesset)
		#print str(keyset)

		if len(fieldset) > 10:
			fieldlist='*'
			nextlist='*'
			prevlist='*'
		else:
			fieldlist = ""
			nextlist = ""
			for f in fieldset:
				if fieldlist != "":
					fieldlist += ","
					nextlist += ","
				if (f != "") and (f[0] != '_'):
					fieldlist += fn + "(" + f + ") as " + f
					nextlist += f
					
				
			#fieldlist= ','.join(fieldset)

		if len(seriesset) > 10:
			seriesset = collapseset(seriesset)
			if len(seriesset) > 10:
				serieslist = '*'
			else:
				serieslist= ""
				for s in serieslist:
					if serieslist != "":
						serieslist += ","
					serieslist += self.database + ".." + s
				#','.join(seriesset)
		else:
			serieslist= ""
			for s in seriesset:
				if serieslist != "":
					serieslist += ","
				serieslist += self.database + ".." + s

		if len(keyset) > 10:
			filter = ""
		else:
			filter = ""
			for vv in keyset:
				if vv == "": 
					continue
				if filter != "":
					filter += " OR "
				filter += "(" + vv + ")"

		out = ""

		#Prefetch Query...
		query = "SELECT " + nextlist + " FROM " + serieslist + " WHERE time < '" + sdtx + "'"

		if filter != "":
			query += " AND ( " + filter + " ) "
		query += " ORDER BY time DESC LIMIT 1"

		self.core.logger.warning(query)
		result = self.db.query(query)
		#print str(result.raw)

		if len(result.raw) > 0:
			for series in result.raw['series']:
				measurement = series['name']
				tags = []
				try:
					tags = series['tags']
				except:
					pass

				shortlist = []

				n = -1
				for addr in addresses:
					n = n+1
					if addr.series == measurement:
						matches = True
						for key in tags:
							#print "Trying " + key
							if key not in addr.tags:
								matches = False
								break
							else:
								#print "Working..."
								if addr.tags[key] != tags[key]:
									matches = False
									break
						if matches == False:
							continue

						#print "Added To Shortlist"
						shortlist.append((addr,n))

			#print str(shortlist)
			#print "Values..."
			for vl in series['values']:
				#print "Checking Values..."
				n=-1
				for xx in vl:
					n= n+1
					col = series['columns'][n]
					if col == 'time':
						tm = xx
						tm = tm[0:10] + ' ' + tm[11:16]
					else:
						for sl in shortlist:
							if sl[0].field == col:
								#value = xx
								#print xx
								#print str(sl[0])
								sl[0].pvalue = xx
								out = out + self.core.AddRecord(n, tm, xx,sl[0].address)
								
		query = "SELECT " + fieldlist + " FROM " + serieslist + " WHERE time >= '" + sdtx + "' AND time <= '" + entx + "'"

		if filter != "":
			query += " AND ( " + filter + " ) "
		query += " GROUP BY time(" + str(chunksize) + "s)" + extragroup + " FILL(previous) ORDER BY time ASC "

		#print query

		result = self.db.query(query)
		self.core.logger.warning(query)

		if len(result.raw) > 0:
			for series in result.raw['series']:
				measurement = series['name']
				tags = []
				try:
					tags = series['tags']
				except:
					pass
				
				shortlist = []

				n = -1
				for addr in addresses:
					n = n+1
					if addr.series == measurement:
						matches = True
						for key in tags:
							#print "Trying " + key
							if key not in addr.tags:
								matches = False
								#print "Failed Match!"
								break
							else:
								#print "Working..."
								if addr.tags[key][0] == "'":
									if addr.tags[key][len(addr.tags[key])-1] == "'":
										addr.tags[key] = addr.tags[key][1:len(addr.tags[key])-1]
								if str(addr.tags[key]) != str(tags[key]):
									matches = False
									#print "Failed Value Match - " + str(addr.tags[key]) + " != " + str(tags[key])
									break
						if matches == False:
							continue

						#print "Added To Shortlist"
						shortlist.append((addr,n))

				for vl in series['values']:
					n=-1
					for xx in vl:
						n = n+1
						col = series['columns'][n]
						if col == 'time':
							tm = xx
							tm = tm[0:10] + ' ' + tm[11:16]
							#tm = tm[0:18] 
						else:
							for sl in shortlist:
								if sl[0].field == col:
									if str(xx) == "None":
										xx = sl[0].pvalue
									else:
										sl[0].pvalue = xx
									
									out = out + self.core.AddRecord(n, tm, xx,sl[0].address)


		#Prefetch Query...
		query = "SELECT " + nextlist + " FROM " + serieslist + " WHERE time > '" + entx + "'"

		if filter != "":
			query += " AND ( " + filter + " ) "
		query += " ORDER BY time ASC LIMIT 1"

		self.core.logger.warning(query)
		result = self.db.query(query)
		#print str(result.raw)

		if len(result.raw) > 0:
			for series in result.raw['series']:
				measurement = series['name']
				tags = []
				try:
					tags = series['tags']
				except:
					pass

				shortlist = []

				n = -1
				for addr in addresses:
					n = n+1
					if addr.series == measurement:
						matches = True
						for key in tags:
							#print "Trying " + key
							if key not in addr.tags:
								matches = False
								break
							else:
								#print "Working..."
								if addr.tags[key] != tags[key]:
									matches = False
									break
						if matches == False:
							continue

						#print "Added To Shortlist"
						shortlist.append((addr,n))

			#print str(shortlist)
			#print "Values..."
			for vl in series['values']:
				#print "Checking Values..."
				n=-1
				for xx in vl:
					n= n+1
					col = series['columns'][n]
					if col == 'time':
						tm = xx
						tm = tm[0:10] + ' ' + tm[11:16]
					else:
						for sl in shortlist:
							if sl[0].field == col:
								sl[0].pvalue = xx
								out = out + self.core.AddRecord(n, tm, xx,sl[0].address)
		print out
		return out.encode('ascii')
class driverfactory:
    def createinstance(self):
        return ardihistdriver()
            
if __name__ == "__main__":
    sdf = driverfactory()
    base = histdriverbase.historian()
    base.start(sdf)