#!/usr/bin/python

import sys
import random
import datetime
import datetime
from datetime import timedelta

from ardi.driver import histdriverbase
from ardi.driver import sqlhelper
import pymssql

def fixdate(dt):
	ps = dt.find('.')
	if ps > -1:
		return dt[0:ps]
	return dt

class ardihistdriver:
    def __init__(self):        
        self.port = 0
        self.db = None        
        self.host = ""
        self.database = ""
        self.username = ""
        self.password = ""
        self.query = ""
        self.stamp = ""
        self.table = ""
        self.lookup = ""
        self.value = ""
        self.filter = ""
        self.datestyle="1"
        self.tz = "local"
        self.options = ""
        self.lookuptable = {}

    def SetAddress(self, addr):
        self.connected = False
        bits = addr.split(":")
        filter = ""

        #print "Raw Address"

        if len(bits) >= 6:
            self.host = bits[0]
            self.port = bits[1]
            self.database = bits[2]
            self.username = bits[3]
            self.password = bits[4]
            self.table = bits[5]
            self.lookup = bits[6]
            self.stamp = bits[7]
            self.value = bits[8]
        if len(bits) > 9:
            self.filter = bits[9]
        if len(bits) > 11:
            self.datestyle = bits[10]
            self.tz = bits[11]
        if len(bits) > 12:
        	self.options = bits[12]

	self.filter = self.filter.replace("{stamp}","s.{stamp}")
	self.filter = self.filter.replace("{lookup}","s.{lookup}")
	self.filter = self.filter.replace("{value}","s.{value}")

	self.filter = self.filter.replace(self.lookup,"s." + self.lookup)
	self.filter = self.filter.replace(self.stamp,"s." + self.stamp)
	self.filter = self.filter.replace(self.value,"s." + self.value)

    def Connect(self):
        if self.db is not None:
        	try:
        		self.Disconnect()
        		self.db = None
        except:
        	traceback.print_exc()
        	pass
        
        try:
        	self.core.logger.info("Connecting To " + self.host + ":" + self.port + " / " + self.database)
        	self.db = pymssql.connect(self.host + ":" + self.port,self.username,self.password,self.database)
        	self.connected = True
        	self.core.logger.info("Successfully Connected!")
        	self.db.close()
        	return True
        except:
        	return False		

    def Disconnect(self):
        try:			
        	self.db.close()
        except:
        	pass        
        
    def Query(self, points, start,end,function,grain,first):
    	self.core.logger.info("Performing '" + function + "' query between " + start + " and " + end) 								

    	sd = datetime.datetime.strptime( start, "%Y-%m-%d %H:%M:%S" )
    	ed = datetime.datetime.strptime( end, "%Y-%m-%d %H:%M:%S" )

		self.core.logger.info("Connecting To " + self.host + ":" + self.port + " / " + self.database)
		db = pymssql.connect(self.host + ":" + self.port,self.username,self.password,self.database)

		sql = sqlhelper.SQLSource(db, self)
		sql.server = "mssql"

		sql.datestyle = self.datestyle
		sql.tz = self.tz
		sql.options = self.options
    	#Note that with appropriate keys, these statements are LESS efficient than the more obvious but theorietically less efficient query...
    	
    	#sql.upperedgequery = "SELECT TOP 1 WITH TIES {stamp},{lookup},{value} FROM {table} WHERE {stamp} <= {start}{additional} ORDER BY ROW_NUMBER() OVER (PARTITION by {lookup} ORDER BY {stamp} DESC)"
    	#sql.loweredgequery = "SELECT TOP 1 WITH TIES {stamp},{lookup},{value} FROM {table} WHERE {stamp} >= {end}{additional} ORDER BY ROW_NUMBER() OVER (PARTITION by {lookup} ORDER BY {stamp} DESC)"

    	#print "Filter: " + filter

		sql.narrowthreshold = 250
		sql.tableprefix = "s."
		sql.upperedgequery = "SELECT s.{stamp},s.{lookup},CAST(s.{value} as FLOAT) FROM {table} s INNER JOIN (SELECT {lookup}, MAX({stamp}) AS {stamp} FROM {table} s1 WHERE {stamp} <= {start} GROUP BY {lookup}) s2 ON s2.{lookup} = s.{lookup} AND s2.{stamp} = s.{stamp}{additional}"
		sql.loweredgequery = "SELECT s.{stamp},s.{lookup},CAST(s.{value} as FLOAT) FROM {table} s INNER JOIN (SELECT {lookup}, MIN({stamp}) AS {stamp} FROM {table} s1 WHERE {stamp} >= {end} GROUP BY {lookup}) s2 ON s2.{lookup} = s.{lookup} AND s2.{stamp} = s.{stamp}{additional}"

		sql.datetimecalc = "DATEADD(s,(CEILING(DATEDIFF(second,{start},{stamp}) / {grain}) * {grain}),{start})"
		sql.numericcalc = "(CEILING(({stamp} - {start}) / {grain}) * {grain}) + {start}"
		#sql.mainquery = "SELECT dt,{lookup},AVG(value) as value FROM (SELECT {extra} as dt,{lookup},{value} as value FROM {table} s WHERE {stamp} > {start} AND {stamp} < {end}{additional} ) as tt GROUP BY dt,{lookup} ORDER BY dt"
		sql.mainquery = "SELECT {extra},{lookup},AVG({value}) FROM {table} s {join} WHERE ( {stamp} > {start} AND {stamp} < {end} ) GROUP BY {extra},{lookup} ORDER BY 1,2"
		sql.rawquery = "SELECT {stamp},{lookup},{value} FROM {table} s {join} WHERE {stamp} > {start} AND {stamp} < {end} ORDER BY {stamp}"
		#sql.mainquery = "SELECT {extra} as ostamp, {lookup} as lookup, AVG({value}) as value FROM {table} WHERE {stamp}>={start} AND {stamp}<={end}{additional} GROUP BY {extra}, {lookup} ORDER BY ostamp,lookup"
		
		sql.postquerylimit = ""
		sql.postselectlimit = " TOP 1"
	
		sql.TableParameters(self.stamp,self.lookup,self.value,self.table,self.filter)
		self.core.logger.info("Calling SQL...")
	
		stime = datetime.datetime.now()
		dt = sql.Fetch(sd, ed, function, grain, points)
		elapsed = (datetime.datetime.now() - stime).total_seconds()
	
		print "Total Time Elapsed: " + str(elapsed)
	
		db.close()
	
		return dt
	
class driverfactory:
    def createinstance(self):
        return ardihistdriver()
            
if __name__ == "__main__":
    sdf = driverfactory()
    base = histdriverbase.historian()
    base.start(sdf)