import sys
import random
import datetime
from datetime import timedelta

from ardi.driver import histdriverbase
from ardi.driver import sqlhelper
import pymssql

def fixdate(dt):
	ps = dt.find('.')
	if ps > -1:
		return dt[0:ps]
	return dt

class ardihistdriver:
    def __init__(self):        
        self.port = 0
        self.db = None        
        self.host = ""
        self.database = ""
        self.username = ""
        self.password = ""
        self.query = ""
        self.stamp = ""
        self.table = ""
        self.lookup = ""
        self.value = ""
        self.filter = ""
        self.datestyle="1"
        self.tz = "local"
        self.lookuptable = {}

    def SetAddress(self, addr):
        self.connected = False
        bits = addr.split(":")
        filter = ""

        #print "Raw Address"

        if len(bits) >= 6:
            self.host = bits[0]
            self.port = bits[1]
            self.database = bits[2]
            self.username = bits[3]
            self.password = bits[4]
            self.table = bits[5]
            self.lookup = bits[6]
            self.stamp = bits[7]
            self.value = bits[8]
        if len(bits) > 9:
            self.filter = bits[9]
        if len(bits) > 11:
            self.datestyle = bits[10]
            self.tz = bits[11]


    def Connect(self):
        if self.db is not None:
        	try:
        		self.Disconnect()
        	except:
        		traceback.print_exc()
        		pass
        try:
        	self.core.logger.info("Connecting To " + self.host + ":" + self.port + " / " + self.database)
        	self.db = pymssql.connect(self.host + ":" + self.port,self.username,self.password,self.database)
        	self.connected = True
        	self.core.logger.info("Successfully Connected!")
        	return True
        except:
        	return False		

    def Disconnect(self):
        try:			
        	self.con.close()
        except:
        	pass        
        
    def Query(self, points, start,end,function,grain,first):
    	self.core.logger.debug("Performing '" + function + "' query on " + str(points) + " between " + start + " and " + end) 								

    	sd = datetime.datetime.strptime( start, "%Y-%m-%d %H:%M:%S" )
    	ed = datetime.datetime.strptime( end, "%Y-%m-%d %H:%M:%S" )

    	try:
    		self.db.commit()
    	except:
    		pass

    	#print points
    	sql = sqlhelper.SQLSource(self.db, self)
    	sql.server = "mssql"

    	sql.datestyle = self.datestyle
    	sql.tz = self.tz
    	sql.upperedgequery = "SELECT TOP 1 WITH TIES {stamp},{lookup},{value} FROM {table} WHERE {stamp} <= {start}{additional} ORDER BY ROW_NUMBER() OVER (PARTITION by {lookup} ORDER BY {stamp} DESC)"
    	sql.datetimecalc = "DATEADD(s,(CEILING(DATEDIFF(second,{start},{stamp}) / {grain}) * {grain}),{start})"
    	sql.numericcalc = "(CEILING(({stamp} - {start}) / {grain}) * {grain}) + {start}"
    	sql.mainquery = "SELECT {extra} as ostamp, {lookup} as lookup, AVG({value}) as value FROM {table} WHERE {stamp}>={start} AND {stamp}<={end}{additional} GROUP BY {extra}, {lookup} ORDER BY ostamp,lookup"
    	sql.loweredgequery = "SELECT TOP 1 WITH TIES {stamp},{lookup},{value} FROM {table} WHERE {stamp} >= {end}{additional} ORDER BY ROW_NUMBER() OVER (PARTITION by {lookup} ORDER BY {stamp} DESC)"

    	sql.TableParameters(self.stamp,self.lookup,self.value,self.table,self.filter)
    	
    	vl = sql.Fetch(sd, ed, function, grain, points)
	print vl
	return vl
  
class driverfactory:
    def createinstance(self):
        return ardihistdriver()
            
if __name__ == "__main__":
    sdf = driverfactory()
    base = histdriverbase.historian()
    base.start(sdf)