import sys
import time
import traceback

import pycurl
import urllib
import json

try:
    from io import BytesIO
except ImportError:
    from StringIO import StringIO as BytesIO

from ardi.driver import drivercore,driverbase

#import mysql.connector
#from mysql.connector import errorcode
#import mysqldb

class sqladdress:
    def __init__(self, addr):
        self.conditionvalue = ""        

class chockinfo:
    def __init__(self):
        self.weight = 0
        self.north = 0
        self.east = 0
        self.floor = 0
        self.roll = 0
        self.pitch = 0
        self.yaw = 0
        self.id = 0
         
class ardiinsdriver:
    def __init__(self):
        self.connected = False
        self.polling = True
        self.pollinterval = 10          
        self.host = ""
        self.version = "1.5.0"
        
        self.offsetx = 0
        self.offsety = 0
        self.offsetz = 0

        self._lwname = ""
        self._east = 0
        self._north = 0
        self._supports = 170
        self._alt = 0
        self._id = 0
        self._mgentry = 0
        self._mgexit = 0
        self._tgentry = 0
        self._tgexit = 0
        self._width = 290;
        self._refreshed = True
        self._multiplier = 1;
        self._heading = 0

        self._lookup = {}
        self._fullupdaterate = 5
        self._counter = self._fullupdaterate

    def SetAddress(self, addr):
        self.connected = False
        bits = addr.split("^")
        filter = ""

        #print "Raw Address"
        self.host = bits[0]
        if len(bits) > 1:
            self.version = bits[1]

        if len(bits) > 2:
            off = bits[2].split(',')
            self.offsetx = float(off[0])
            self.offsety = float(off[1])
            self.offsetz = float(off[2])
        #if len(bits) > 3:
        #    self.host = self.host + ':' + bits[1]
        #    self.version = bits[2]

    def Connect(self):
        try:            
            buffer = BytesIO()
            ar = pycurl.Curl()

            for pnt in self.core.pointlist:            
                self._lookup[pnt.address] = pnt.code

            self.core.logger.info("Requesting INS Configuration @ " + 'http://' + self.host + '/api/v' + self.version + '/json/panelinfo')

            ar.setopt(ar.URL, 'http://' + self.host + '/api/v' + self.version + '/json/panelinfo')
            ar.setopt(ar.WRITEFUNCTION, buffer.write)
            ar.perform()        

            # HTTP response code, e.g. 200.
            if ar.getinfo(ar.RESPONSE_CODE) == 200:

                data = json.loads(buffer.getvalue())
                #print str(data)
                self._width = data['width']
                self._supports = data['supports']
                self._north = data['northing']
                self._east = data['easting']
                self._heading = data['heading']
                #self._alt = data['altitude']

                self.core.logger.info("Connected to " + data["name"] + " - " + str(self._supports) + " supports and " + str(self._width) + "m long, heading " + str(self._heading))
                self.core.logger.info("Calibration Point: " + str(self._north) + ", " + str(self._east))
                self.connected = True
                ar.close()
                return True
            else:
                print "Request Failed"
                self.connected = False
                ar.close()
        except Exception as e:
            print "Exception Occured - " + str(e)
            self.connected = False;
            return False
        return True

    def Optimise(self):
        return True

    def Disconnect(self):
        self.connected = False
        
    def Poll(self):
        #print "Attempting To Poll..."
        try:
            buffer = BytesIO()
            ar = pycurl.Curl()
            ar.setopt(ar.URL, 'http://' + self.host + '/api/v' + self.version + '/json/cutmodel/-1')
            ar.setopt(ar.WRITEFUNCTION, buffer.write)
            ar.perform()

            #print "Sending..."

            # HTTP response code, e.g. 200.
            if ar.getinfo(ar.RESPONSE_CODE) == 200:
                data = json.loads(buffer.getvalue())
                self.core.logger.info("Profile " + str(data['id']) + " contains " + str(data['profileSize']) + " samples")

                chockset = {}
                for sup in range(0,self._supports+2):
                    chockset[sup] = chockinfo()
                    chockset[sup].id = sup

                indx = 0
                pro = data["profile"]
                while indx < len(pro):
                    support = (pro[indx] / self._width) * (self._supports)

                    aweight = support % 1
                    sup = int(support)
                    chockset[sup].weight += aweight
                    chockset[sup].north += pro[indx+5] * aweight
                    chockset[sup].east += pro[indx+4] * aweight
                    chockset[sup].roll += pro[indx+1] * aweight
                    chockset[sup].pitch += pro[indx+2] * aweight
                    chockset[sup].yaw += pro[indx+3] * aweight
                    chockset[sup].floor += pro[indx+6] * aweight

                    aweight = 1-aweight
                    if (aweight > 0) and (support < self._supports):
                        chockset[sup].weight += aweight
                        chockset[sup].north += pro[indx+5] * aweight
                        chockset[sup].east += pro[indx+4] * aweight
                        chockset[sup].floor += pro[indx+6] * aweight
                        
                    #print str(pro[indx]) + " = " + str(pro[indx+4]) + "," + str(pro[indx + 5])
                    #print str(support) + " = " + str(pro[indx+4]) + "," + str(pro[indx + 5])
                    indx = indx + 8
                ar.close()
            else:
                self.core.logger.warning("No Response - Check Driver Version")
                ar.close()

            #print str(chockset)
            #print str(self._lookup)

            for chkid,chk in chockset.items():
                if chk.weight > 0:
                    chk.north /= chk.weight
                    chk.east /= chk.weight
                    chk.roll /= chk.weight
                    chk.pitch /= chk.weight
                    chk.yaw /= chk.weight
                    chk.floor /= chk.weight
                    
                    nm = "position[" + str(chkid+1) + "]"
                    if nm in self._lookup:
                        self.core.UpdateData(self._lookup[nm],str(chk.east + self.offsetx) + "," + str(chk.north + self.offsety) + "," + str(chk.floor + self.offsetz))
                        #self.core.logger.info("Position: " + nm + " = " + str(chk.east) + "," + str(chk.north))
                        
                    nm = "rotation[" + str(chkid+1) + "]"
                    if nm in self._lookup:
                        #vpos = str(chk.pitch) + "," + str(225 + chk.yaw) + "," + str(chk.roll)
                        vpos = str(chk.pitch) + "," + str(self._heading - chk.yaw) + "," + str(chk.roll)
                        #vpos = str(chk.roll) + "," + str((chk.pitch) + 95) + "," + str(chk.yaw)
                        self.core.UpdateData(self._lookup[nm],vpos)
                        #print str(chkid) + " : " + str(chk.roll) + ", " + str(chk.pitch) + "," + str(chk.yaw)
                
        except Exception as ex:
            traceback.print_exc()
            print "Data Aquisition Failed - " + str(ex)
            pass

class driverfactory:
    def createinstance(self):
        return ardiinsdriver()

if __name__ == "__main__":
    sdf = driverfactory()
    base = driverbase.ardidriver()
    base.start(sdf)
