import sys

from ardi.driver import drivercore,driverbase

from influxdb import InfluxDBClient
import traceback

class sqlquery:
    def __init__(self):
        self.series = ""
        self.fields = []
        self.codes = []
        self.qstring = ""

    def build(self):
        self.qstring = "SELECT "
        first = True
        for a in self.fields:
            if first == True:
                first = False
            else:
                self.qstring += ","
            self.qstring += a    
        self.qstring += " FROM " + self.series + " ORDER BY time DESC LIMIT 1"
        print "Build Query String: " + self.qstring

class influxaddress:
    def __init__(self, addr):
        self.series = ""
        self.field = ""
        self.tags = {}
        self.condition = ""
        self.address = addr

        print "Parsing Address: " + addr
        
        bits = addr.split('|')
        if len(bits)>=3:
            self.field = bits[0]
            self.series = bits[2]
            self.condition = bits[1]
            tg = bits[1].split(',')
            for q in tg:
                v = q.split('=')
                self.tags[v[0]] = v[1]
        else:
            raise ValueError('Invalid Address - Empty or Too Generic') 
                            
class ardiinfluxdriver:
    def __init__(self):
        self.connected = False
        self.polling = True
        self.pollinterval = 5
        self.port = 0
        self.db = None        
        self.host = ""
        self.database = ""
        self.username = ""
        self.password = ""
        self.queries = ()
        self.defaultseries = ""
        self.filter = ""
        self.addresses = []

    def SetAddress(self, addr):
        self.connected = False
        bits = addr.split(":")        
        if len(bits) >= 5:
            self.host = bits[0]
            self.port = bits[1]
            self.database = bits[2]
            self.username = bits[3]
            self.password = bits[4]
            self.defaultseries = bits[5]
            self.filter = bits[6]
    
    def Connect(self):
        try:
            #print "Connecting..."
            #self.db = mysql.connector.connect(user=self.username,password=self.password,host=self.host,database=self.database)
            self.db = InfluxDBClient(self.host, self.port,self.username,self.password,self.database)#MySQLdb.connect(self.host,self.username,self.password,self.database)
            self.connected = True
        except:
            #print "Error Connecting to Database!"
            traceback.print_exc()
            return False
        return True

    def Optimise(self):

        #OK - collect all of the addresses with a common table...
        try:

            keyset = []
            seriesset = []
            fieldset = []

            #addresses = []
            extragroup = ""
            tagged = ()

            for p in self.core.pointlist:
                try:
                    ad = influxaddress(p.address)
                    self.addresses.append(ad)
                    #p.addr = ad
                    if ad.field not in fieldset:
                        fieldset.append(ad.field)
                    if ad.series == None or ad.series == "":
                        ad.series = self.defaultseries
                    if ad.series not in seriesset:
                        seriesset.append(ad.series)
                    if ad.condition not in keyset:
                        keyset.append(ad.condition)
                        for key in ad.tags:
                            if key not in fieldset:
                                if extragroup != "":
                                    extragroup += ","
                                extragroup += key
                            if key not in fieldset:
                                fieldset.append(key) 
                except:
                    pass

            #if len(fieldset) > 10:
            #    fieldlist='*'
            #else:
            fieldlist = ""
            for f in fieldset:
                if fieldlist != "":
                    fieldlist += ","
                if f[0] != '_':
                    fieldlist += f
                    
                #fieldlist= ','.join(fieldset)

            if len(seriesset) > 10:
                seriesset = collapseset(seriesset)
                if len(seriesset) > 10:
                    serieslist = '*'
                else:
                    serieslist= ""
                    for s in serieslist:
                        if serieslist != "":
                            serieslist += ","
                        serieslist += self.database + ".." + s
                    #','.join(seriesset)
            else:
                serieslist= ""
                for s in seriesset:
                    if serieslist != "":
                        serieslist += ","
                    serieslist += self.database + ".." + s

            if len(keyset) > 10:
                filter = ""
            else:
                filter = ""
                for vv in keyset:
                    if filter != "":
                        filter += " OR "
                    filter += "(" + vv + ")"

            query = "SELECT " + fieldlist + " FROM " + serieslist + ""

            if filter != "":
                query += " WHERE ( " + filter + " ) "
                
            if len(keyset) > 0:
                query += " GROUP BY "
                query += extragroup
            
            query += " ORDER BY time DESC"
            
            query += " LIMIT 1"

            self.query = query
        except:
            traceback.print_exc()

        self.core.logger.warning("Settled On Query: " + self.query)

        return

        self.queries = []

        try:
            for itm in self.core.pointlist:
                addr = sqladdress(itm.address)
                if addr.series == "":
                    addr.series = self.defaultseries
                    
                found = False            
                for existing in self.queries:
                    if (addr.series == existing.series):
                                                                
                        found = True
                        existing.fields = existing.fields + [addr.field]
                        #existing.codes += (itm.code,)
                        break
                if found == False:
                    s = sqlquery()
                    
                    s.series = addr.series
                    s.fields += [addr.field]
                    
                    self.queries.append(s)
        except:
            traceback.print_exc()

        #Build Query Strings...
        for itm in self.queries:
            itm.build()

    def Disconnect(self):
        self.connected = False
        self.db.close()
        
    def Poll(self):
        #for itm in self.queries:
            if True:#try:
                result = self.db.query(self.query)

                points = result.get_points()
                lst = list(points)
                #print "Row: " + str(lst)
                
                for px in lst:
                    vx = -1
                    for addr in self.addresses:
                        vx = vx + 1
                        passit = True
                        for key in addr.tags:
                            #print "Comparing " + str(addr) + "..."
                            if key not in px:
                                passit = False
                                #print "Key " + key + " Not Present"
                                continue
                            dv = str(addr.tags[key])
                            if dv[0] == '"':
                                dv = dv[1:len(dv)-1]
                            if dv[0] == '\'':
                                dv = dv[1:len(dv)-1]
                            if str(px[key]) != dv:
                                #print "Value " + str(px[key]) + " <> " + str(dv)
                                passit = False
                                #continue
                            
                        #print str(px)
                        if passit == True:
                            val = px[addr.field]
                            #print "Passed Check! This is " + str(self.core.pointlist[vx])
                            self.core.UpdateData(self.core.pointlist[vx].code,str(val))
                            print "Data Updated - " + self.addresses[vx].field + ' = ' + str(val)
                        #else:
                            #print "Test Failed"
            #except:
            #    print "Failure!"                    

class driverfactory:
    def createinstance(self):
        return ardiinfluxdriver()
            
if __name__ == "__main__":
    sdf = driverfactory()
    base = driverbase.ardidriver()
    base.start(sdf)