import sys

from ardi.driver import drivercore,driverbase

from influxdb import InfluxDBClient
import traceback

class sqlquery:
    def __init__(self):
        self.series = ""
        self.fields = []
        self.codes = []
        self.qstring = ""

    def build(self):
        self.qstring = "SELECT "
        first = True
        for a in self.fields:
            if first == True:
                first = False
            else:
                self.qstring += ","
            self.qstring += a[0]        
        self.qstring += " FROM " + self.series + " ORDER BY time DESC LIMIT 1"
        print "Build Query String: " + self.qstring

class sqladdress:
    def __init__(self, addr):
        self.series = ""
        self.field = ""
        
        bits = addr.split('|')
        if len(bits)>=2:
            self.field = bits[0]
            self.series = bits[1]            
                            
class ardiinfluxdriver:
    def __init__(self):
        self.connected = False
        self.polling = True
        self.pollinterval = 5
        self.port = 0
        self.db = None        
        self.host = ""
        self.database = ""
        self.username = ""
        self.password = ""
        self.queries = ()
        self.defaultseries = ""
        self.filter = ""

    def SetAddress(self, addr):
        self.connected = False
        bits = addr.split(":")        
        if len(bits) >= 5:
            self.host = bits[0]
            self.port = bits[1]
            self.database = bits[2]
            self.username = bits[3]
            self.password = bits[4]
            self.defaultseries = bits[5]
            self.filter = bits[6]
    
    def Connect(self):
        try:
            #print "Connecting..."
            #self.db = mysql.connector.connect(user=self.username,password=self.password,host=self.host,database=self.database)
            self.db = InfluxDBClient(self.host, self.port,self.username,self.password,self.database)#MySQLdb.connect(self.host,self.username,self.password,self.database)
            self.connected = True
        except:
            #print "Error Connecting to Database!"
            traceback.print_exc()
            return False
        return True

    def Optimise(self):

        #OK - collect all of the addresses with a common table...
        self.queries = ()

        print "Optimising!"

        try:
            for itm in self.core.pointlist:
                addr = sqladdress(itm.address)
                if addr.series == "":
                    addr.series = self.defaultseries
                    
                found = False            
                for existing in self.queries:
                    if (addr.series == existing.series):
                                                                
                        found = True
                        existing.fields = existing.fields + ((addr.field,itm.code),)
                        #existing.codes += (itm.code,)
                        break
                if found == False:
                    s = sqlquery()
                    
                    s.series = addr.series
                    s.fields += ((addr.field,itm.code),)
                    #s.codes += (itm.code,)
                    
                    self.queries = self.queries + (s,)
        except:
            traceback.print_exc()

        #Build Query Strings...
        for itm in self.queries:
            itm.build()

    def Disconnect(self):
        self.connected = False
        self.db.close()
        
    def Poll(self):
        for itm in self.queries:
            try:
                result = self.db.query(itm.qstring)
                points = result.get_points()
                for px in points:
                    for n in itm.fields:
                        try:
                            val = px[n[0]]
                            self.core.UpdateData(n[1],str(val))
                            print "Data Updated - " + n[0] + ' = ' + str(val)
                        except:
                            pass
                #print str(list(points))
            except:
                print "Failure!"                    

class driverfactory:
    def createinstance(self):
        return ardiinfluxdriver()
            
if __name__ == "__main__":
    sdf = driverfactory()
    base = driverbase.ardidriver()
    base.start(sdf)