import sys
import time
import traceback

from ardi.driver import drivercore,driverbase
import MySQLdb

#import mysql.connector
#from mysql.connector import errorcode
#import mysqldb

class sqladdress:
    def __init__(self, addr):
        self.conditionvalue = ""        
                            
class ardiinsdriver:
    def __init__(self):
        self.connected = False
        self.polling = True
        self.pollinterval = 10        
        self.port = 0
        self.db = None        
        self.host = ""
        self.database = ""
        self.username = ""
        self.password = ""
        self.prefix = "lasc_"

        self._lwname = ""
        self._east = 0
        self._north = 0
        self._supports = 170
        self._alt = 0
        self._id = 0
        self._mgentry = 0
        self._mgexit = 0
        self._tgentry = 0
        self._tgexit = 0
        self._width = 290;
        self._refreshed = True
        self._multiplier = 1;

        self._lookup = {}
        self._fullupdaterate = 5
        self._counter = self._fullupdaterate

    def SetAddress(self, addr):
        self.connected = False
        bits = addr.split(":")
        filter = ""

        #print "Raw Address"

        if len(bits) >= 5:
            self.host = bits[0]
            self.port = bits[1]
            self.database = bits[2]
            self.username = bits[3]
            self.password = bits[4]
            if len(bits) > 5:
                self.prefix = bits[5]
    
    def Connect(self):
        try:            
            self.db = MySQLdb.connect(self.host,self.username,self.password,self.database)
            self.connected = True        
        except StandardError as err:
            self.core.logger.error(err[1])
            return False
        except:            
            return False
        return True

    def Optimise(self):
        #Perform the initial data lookup for this longwall INS
        csr = self.db.cursor()
        timing = time.time()            
        csr.execute("SELECT eastingRef, northingRef,altitudeRef,numSupports,id,name,mgEntry,mgExit,tgEntry,tgExit,width FROM " + self.prefix + "panelinfo")                
        for row in csr:
            self._lwname = row[5]
            self._east = float(row[0])
            self._north = float(row[1])
            self._supports = row[3]
            self._alt = float(row[2])
            self._id = row[4]
            self._mgentry = row[6]
            self._mgexit = row[7]
            self._tgentry = row[8]
            self._tgexit = row[9]
            self._width = row[10];

        self._north = 0
        self._east = 0

        self.db.commit()
        csr.close()

        #Prepare lookup table (optimisation)
        for pnt in self.core.pointlist:            
            self._lookup[pnt.address] = pnt.code

        self.core.logger.info('Longwall ' + self._lwname + " has " + str(self._supports) + " roof supports")        

        self.refreshed = True

    def Disconnect(self):
        self.connected = False
        self.db.close()
        
    def Poll(self):
        try:

            if self.refreshed == True:
                #Set the basics
                self.refreshed = False
                if "supports" in self._lookup:
                    self.core.UpdateData(self._lookup["supports"],str(self._supports))
                if "name" in self._lookup:                    
                    self.core.UpdateData(self._lookup["name"],str(self._lwname))

            csr = self.db.cursor()

            qry = "SELECT UtmEasting,UtmNorthing,Altitude, PitchAngle,RollAngle,CantAngle,TrueHeadingAngle FROM " + self.prefix + "insinstant"
            #print qry
            try:
                csr.execute(qry)
                for row in csr:
                    nm = "shearer_position"                
                    if nm in self._lookup:
                        ps = str(row[0]) + "," + str(row[1]) + "," + str(row[2])
                        self.core.UpdateData(self._lookup[nm],ps)
                        
                    nm = "shearer_rotation"                
                    if nm in self._lookup:
                        ps = str(row[3]) + "," + (str(90 - row[6])) + "," + str(row[5])
                        self.core.UpdateData(self._lookup[nm],ps)
                        #print ps
            except:
                traceback.print_exc()
                print "Failed To Get Current Location"
                pass
            csr.close();

            self._counter = self._counter + 1
            if self._counter < self._fullupdaterate:
                return

            self._counter = 0

            self.UpdateLastProfile()
            
        except StandardError as err:
            try:
                if (str(err[1]) == "MySQL server has gone away") or (str(err[1]) == ""):
                    self.core.logger.error('Data Connection Lost')
                    try:
                        self.db.close()
                    except:
                        pass
                    self.connected = False
                    return
                self.core.logger.error('Query Failed: ' + err[1])
            except Exception as e:
                self.core.logger.error('Failure - ' + str(err))
                traceback.print_exc()

    def UpdateLastProfile(self):
        csr = self.db.cursor()
        timing = time.time()

        #OK - get the most recent profile ID...
        csr.execute("SELECT profileid,startid, endid FROM " + self.prefix + "profiledata ORDER BY profileid DESC LIMIT 2" )
        
        profilestart = 0
        profileend = 0
        profileid = 0
        
        for row in csr:
            if profilestart == 0:
                profilestart = row[1]
                profileend = row[2]
                profileid = row[0]

        self.db.commit()                     
        csr.close()

        #Now, fetch the details on a per-chock basis...
        csr = self.db.cursor()

        qry = "SELECT AVG(RollAngle),AVG(PitchAngle),AVG(CantAngle),AVG(Altitude),AVG(UtmNorthing),AVG(UtmEasting), CEILING((" + str(self._supports) + "*(((ShearerOdometry-" + str(self._mgentry) + ")+1) / " + str(self._width) + "))) as ShearerID, AVG(TrueHeadingAngle) FROM `lasc_ln270data` WHERE id >= " + str(profilestart) + " AND id <= " + str(profileend) + " GROUP BY ShearerID"
        csr.execute(qry)
        
        for row in csr:
            if row[6] > self._supports:
                continue                
            
            nm = "roll[" + str(row[6]) + "]"                
            if nm in self._lookup:
                self.core.UpdateData(self._lookup[nm],str(row[0]))
                
            nm = "pitch[" + str(row[6]) + "]"
            if nm in self._lookup:
                self.core.UpdateData(self._lookup[nm],str(row[1]))                            
                
            nm = "cant[" + str(row[6]) + "]"
            if nm in self._lookup:
                self.core.UpdateData(self._lookup[nm],str(row[2]))

            nm = "position[" + str(row[6]) + "]"
            if nm in self._lookup:
                vpos = str((float(row[5]) - self._east)) + "," + str((float(row[4]) - self._north)) + "," + str((float(row[3]) - self._alt))
                self.core.UpdateData(self._lookup[nm],vpos)
                #print vpos

            nm = "rotation[" + str(row[6]) + "]"
            if nm in self._lookup:
                vpos = "0," + str(90 - (float(row[7]))) + ",0"
                self.core.UpdateData(self._lookup[nm],vpos)
                #print vpos

        self.db.commit()
        csr.close()

class driverfactory:
    def createinstance(self):
        return ardiinsdriver()

if __name__ == "__main__":
    sdf = driverfactory()
    base = driverbase.ardidriver()
    base.start(sdf)
