import sys
import random
import os

from ardi.driver import drivercore, driverbase

class ardiranddriver:
    def __init__(self):
        self.connected = False
        self.polling = True
        self.path = "/etc/openvpn/openvpn-status.log";
    
    def SetAddress(self, addr):
        self.connected = False
        self.path = addr;

    def GetStatusFile(self):
        statpath = "/etc/openvpn/server.conf"
        if !os.path.isfile(statpath):
            return ""

        with open(statpath) as f:
            for line in f:
                pieces = line.split(' ')
                if pieces[0] == 'status':
                    if pieces[0][0] == '/':
                        return pieces[1:].join(' ')
                    else:
                        return "/etc/openvpn/" + pieces[1:].join(' ')
        return ""

    def Connect(self):
        if os.path.isfile(self.path):
            self.connected = True
            core.logging.info("Connected To " + str(self.path))
            return True

        core.logging.warning("Invalid path specified in driver path, checking OpenVPN configuration...")
        sst = self.GetStatusFile();
        if sst != "":
            self.path = sst
            if os.path.isfile(self.path):
                core.logging.warning("Switched to default openvpn status file from configuration")
                self.connected = True
                return True
            
        core.logging.error("Unable To Connect - File Does Not Exist")
        self.connected = False
        return False;
    
    def Disconnect(self):
        self.connected = False
        
    def Optimise(self):
        pass
        
    def Poll(self):

        hits = []
        with open(self.path) as f:
            for line in f:
                pieces = line.split(',')
                if pieces[0] == 'OpenVPN CLIENT LIST':
                    continue
                if pieces[0] == 'Updated':
                    continue
                if pieces[0] == 'Common Name':
                    continue
                if pieces[0] == 'ROUTING TABLE':
                    break;
                hits.append(pieces[0])
        
        for point in self.core.pointlist:
            try:
                val = 0
                if point.addr in hits:
                    val = 1
                self.core.UpdateData(point.code,str(val))                
            except:
                pass

class driverfactory:
    def createinstance(self):
        return ardiranddriver()
            
if __name__ == "__main__":
    sdf = driverfactory()
    base = driverbase.ardidriver()
    base.start(sdf)
    